import { LRLanguage, LanguageSupport } from '@codemirror/language';
import * as _codemirror_state from '@codemirror/state';
import { EditorState, StateField, Extension } from '@codemirror/state';
import { CompletionContext, CompletionResult } from '@codemirror/autocomplete';
import { SyntaxNode } from '@lezer/common';
import { Tooltip } from '@codemirror/view';

/**
 * Represents a variable or a property of an object
 */
interface ELIdentifier {
    name: string;
    detail?: string;
    info?: string;
    type?: string[];
}
/**
 * Represents a function or a method of an object
 */
interface ELFunction {
    name: string;
    args?: ELParameter[];
    info?: string;
    returnType?: string[];
}
interface ELParameter {
    name: string;
    type?: string[];
    info?: string;
    optional?: boolean;
}
interface ELType {
    identifiers?: ELIdentifier[];
    functions?: ELFunction[];
    info?: string;
}
interface ExpressionLanguageConfig {
    types?: {
        [key: string]: ELType;
    };
    identifiers?: ELIdentifier[];
    functions?: ELFunction[];
}
interface ELKeyword {
    name: string;
    detail?: string;
    info?: string;
}
declare enum ELScalar {
    Bool = "bool",
    Number = "number",
    String = "string",
    Null = "null",
    Any = "any"
}

declare function expressionLanguageCompletion(context: CompletionContext): CompletionResult | null;

declare const complete_d_expressionLanguageCompletion: typeof expressionLanguageCompletion;
declare namespace complete_d {
  export { complete_d_expressionLanguageCompletion as expressionLanguageCompletion };
}

declare const expressionLanguageLinter: _codemirror_state.Extension;

declare const linter_d_expressionLanguageLinter: typeof expressionLanguageLinter;
declare namespace linter_d {
  export { linter_d_expressionLanguageLinter as expressionLanguageLinter };
}

declare const Function: number;
declare const Method: number;
declare const Property: number;
declare const Variable: number;

declare const createInfoElement: (html: string) => HTMLDivElement;
declare function resolveFunctionDefinition(node: SyntaxNode | null, state: EditorState, config: ExpressionLanguageConfig): ELFunction | undefined;
declare const resolveIdentifier: (nodeTypeId: typeof Method | typeof Property | typeof Function | typeof Variable, identifier?: string, config?: {
    identifiers?: ELIdentifier[];
    functions?: ELFunction[];
}) => ELIdentifier | ELFunction | undefined;
declare function resolveTypes(state: EditorState, node: SyntaxNode | undefined | null, config: ExpressionLanguageConfig): Set<string>;
declare function getExpressionLanguageConfig(state: EditorState): ExpressionLanguageConfig;
declare const keywords: ELKeyword[];

declare const utils_d_createInfoElement: typeof createInfoElement;
declare const utils_d_getExpressionLanguageConfig: typeof getExpressionLanguageConfig;
declare const utils_d_keywords: typeof keywords;
declare const utils_d_resolveFunctionDefinition: typeof resolveFunctionDefinition;
declare const utils_d_resolveIdentifier: typeof resolveIdentifier;
declare const utils_d_resolveTypes: typeof resolveTypes;
declare namespace utils_d {
  export { utils_d_createInfoElement as createInfoElement, utils_d_getExpressionLanguageConfig as getExpressionLanguageConfig, utils_d_keywords as keywords, utils_d_resolveFunctionDefinition as resolveFunctionDefinition, utils_d_resolveIdentifier as resolveIdentifier, utils_d_resolveTypes as resolveTypes };
}

declare const cursorTooltipField: StateField<readonly Tooltip[]>;
declare const keywordTooltip: _codemirror_state.Extension & {
    active: StateField<readonly Tooltip[]>;
};

declare const tooltip_d_cursorTooltipField: typeof cursorTooltipField;
declare const tooltip_d_keywordTooltip: typeof keywordTooltip;
declare namespace tooltip_d {
  export { tooltip_d_cursorTooltipField as cursorTooltipField, tooltip_d_keywordTooltip as keywordTooltip };
}

declare const ELLanguage: LRLanguage;
declare function expressionlanguage(config?: ExpressionLanguageConfig, extensions?: Extension[]): LanguageSupport;

export { type ELFunction, type ELIdentifier, type ELKeyword, ELLanguage, type ELParameter, ELScalar, type ELType, type ExpressionLanguageConfig, complete_d as _complete, linter_d as _linter, tooltip_d as _tooltip, utils_d as _utils, expressionlanguage };
